@@ -160,20 +160,20 @@ fn get_existing_payees() -> &'static [&'static str] {
 
 fn get_render_config() -> RenderConfig {
     let mut render_config = RenderConfig::default();
-    render_config.prompt_prefix = Styled::new("$").with_fg(Color::Red);
-    render_config.highlighted_option_prefix = Styled::new("➠").with_fg(Color::Yellow);
-    render_config.selected_checkbox = Styled::new("☑").with_fg(Color::Green);
+    render_config.prompt_prefix = Styled::new("$").with_fg(Color::LightRed);
+    render_config.highlighted_option_prefix = Styled::new("➠").with_fg(Color::LightYellow);
+    render_config.selected_checkbox = Styled::new("☑").with_fg(Color::LightGreen);
     render_config.scroll_up_prefix = Styled::new("⇞");
     render_config.scroll_down_prefix = Styled::new("⇟");
     render_config.unselected_checkbox = Styled::new("☐");
 
     render_config.error_message = render_config
         .error_message
-        .with_prefix(Styled::new("❌").with_fg(Color::Red));
+        .with_prefix(Styled::new("❌").with_fg(Color::LightRed));
 
     render_config.answer = StyleSheet::new()
         .with_attr(Attributes::ITALIC)
-        .with_fg(Color::Yellow);
+        .with_fg(Color::LightYellow);
 
     render_config.help_message = StyleSheet::new().with_fg(Color::DarkYellow);
 
