@@ -1,4 +1,4 @@
-use inquire::{length, required, ui::RenderConfig, validator::InquireLength, Text};
+use inquire::{length, required, ui::RenderConfig, Text};
 
 fn main() {
     let answer = Text::new("What's your name?")
