@@ -4,6 +4,7 @@
 //! for information via the CLI. With `inquire`, you can use:
 //!
 //! - [`Text`] to get text input from the user, with _built-in auto-completion support_;
+//! - [`Editor`]* to get longer text inputs by opening a text editor for the user;
 //! - [`DateSelect`]* to get a date input from the user, selected via an _interactive calendar_;
 //! - [`Select`] to ask the user to select one option from a given list;
 //! - [`MultiSelect`] to ask the user to select an arbitrary number of options from a given list;
@@ -25,6 +26,7 @@
 //!   - Auto-completion for [`Text`] prompts;
 //!   - Custom list filters for Select and [`MultiSelect`] prompts;
 //!   - Custom parsers for [`Confirm`] and [`CustomType`] prompts;
+//!   - Custom extensions for files created by [`Editor`] prompts;
 //!   - and many others!
 //!
 //! \* Date-related features are available by enabling the `date` feature.
@@ -32,11 +34,17 @@
 //! # Simple Example
 //!
 //! ```rust no_run
-//! use inquire::{max_length, validator::InquireLength, Text};
+//! use inquire::{Text, validator::StringValidator};
 //!
 //! fn main() {
+//!     let validator: StringValidator = &|input| if input.chars().count() > 140 {
+//!         Err(String::from("You're only allowed 140 characters."))
+//!     } else {
+//!         Ok(())
+//!     };
+//!
 //!     let status = Text::new("What are you thinking about?")
-//!         .with_validator(max_length!(140, "You're only allowed 140 characters."))
+//!         .with_validator(validator)
 //!         .prompt();
 //!     
 //!     match status {
@@ -53,8 +61,10 @@
 //! [`Confirm`]: crate::Confirm
 //! [`CustomType`]: crate::CustomType
 //! [`Password`]: crate::Password
+//! [`Editor`]: crate::Editor
 
 #![warn(missing_docs)]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 pub mod config;
 #[cfg(feature = "date")]
@@ -65,6 +75,7 @@ mod input;
 pub mod list_option;
 pub mod parser;
 mod prompts;
+mod terminal;
 pub mod ui;
 mod utils;
 pub mod validator;
