@@ -1,4 +1,4 @@
-use std::{collections::HashSet, fmt::Display};
+use std::{collections::BTreeSet, fmt::Display};
 
 use crate::{
     config::{self, Filter},
@@ -6,9 +6,8 @@ use crate::{
     formatter::MultiOptionFormatter,
     input::Input,
     list_option::ListOption,
-    ui::{
-        crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, MultiSelectBackend, RenderConfig,
-    },
+    terminal::get_default_terminal,
+    ui::{Backend, Key, KeyModifiers, MultiSelectBackend, RenderConfig},
     utils::paginate,
     validator::MultiOptionValidator,
 };
@@ -34,6 +33,7 @@ use crate::{
 /// - **Validator**: Custom validator to make sure a given submitted input pass the specified requirements, e.g. not allowing 0 selected options or limiting the number of options that the user is allowed to select.
 ///   - No validators are on by default.
 /// - **Page size**: Number of options displayed at once, 7 by default.
+/// - **Display option indexes**: On long lists, it might be helpful to display the indexes of the options to the user. Via the `RenderConfig`, you can set the display mode of the indexes as a prefix of an option. The default configuration is `None`, to not render any index when displaying the options.
 /// - **Filter function**: Function that defines if an option is displayed or not based on the current filter input.
 /// - **Keep filter flag**: Whether the current filter input should be cleared or not after a selection is made. Defaults to true.
 ///
@@ -82,6 +82,13 @@ pub struct MultiSelect<'a, T> {
     pub validator: Option<MultiOptionValidator<'a, T>>,
 
     /// RenderConfig to apply to the rendered interface.
+    ///
+    /// Note: The default render config considers if the NO_COLOR environment variable
+    /// is set to decide whether to render the colored config or the empty one.
+    ///
+    /// When overriding the config in a prompt, NO_COLOR is no longer considered and your
+    /// config is treated as the only source of truth. If you want to customize colors
+    /// and still suport NO_COLOR, you will have to do this on your end.
     pub render_config: &'a RenderConfig,
 }
 
@@ -244,6 +251,13 @@ where
     }
 
     /// Sets the provided color theme to this prompt.
+    ///
+    /// Note: The default render config considers if the NO_COLOR environment variable
+    /// is set to decide whether to render the colored config or the empty one.
+    ///
+    /// When overriding the config in a prompt, NO_COLOR is no longer considered and your
+    /// config is treated as the only source of truth. If you want to customize colors
+    /// and still suport NO_COLOR, you will have to do this on your end.
     pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
         self.render_config = render_config;
         self
@@ -252,19 +266,58 @@ where
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     ///
-    /// Returns the owned object selected by the user.
+    /// Returns the owned objects selected by the user.
+    ///
+    /// This method is intended for flows where the user skipping/cancelling
+    /// the prompt - by pressing ESC - is considered normal behavior. In this case,
+    /// it does not return `Err(InquireError::OperationCanceled)`, but `Ok(None)`.
+    ///
+    /// Meanwhile, if the user does submit an answer, the method wraps the return
+    /// type with `Some`.
+    pub fn prompt_skippable(self) -> InquireResult<Option<Vec<T>>> {
+        match self.prompt() {
+            Ok(answer) => Ok(Some(answer)),
+            Err(InquireError::OperationCanceled) => Ok(None),
+            Err(err) => Err(err),
+        }
+    }
+
+    /// Parses the provided behavioral and rendering options and prompts
+    /// the CLI user for input according to the defined rules.
+    ///
+    /// Returns the owned objects selected by the user.
     pub fn prompt(self) -> InquireResult<Vec<T>> {
         self.raw_prompt()
             .map(|op| op.into_iter().map(|o| o.value).collect())
     }
 
+    /// Parses the provided behavioral and rendering options and prompts
+    /// the CLI user for input according to the defined rules.
+    ///
+    /// Returns a vector of [`ListOption`](crate::list_option::ListOption)s containing
+    /// the index of the selections and the owned objects selected by the user.
+    ///
+    /// This method is intended for flows where the user skipping/cancelling
+    /// the prompt - by pressing ESC - is considered normal behavior. In this case,
+    /// it does not return `Err(InquireError::OperationCanceled)`, but `Ok(None)`.
+    ///
+    /// Meanwhile, if the user does submit an answer, the method wraps the return
+    /// type with `Some`.
+    pub fn raw_prompt_skippable(self) -> InquireResult<Option<Vec<ListOption<T>>>> {
+        match self.raw_prompt() {
+            Ok(answer) => Ok(Some(answer)),
+            Err(InquireError::OperationCanceled) => Ok(None),
+            Err(err) => Err(err),
+        }
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     ///
     /// Returns a [`ListOption`](crate::list_option::ListOption) containing
     /// the index of the selection and the owned object selected by the user.
     pub fn raw_prompt(self) -> InquireResult<Vec<ListOption<T>>> {
-        let terminal = CrosstermTerminal::new()?;
+        let terminal = get_default_terminal()?;
         let mut backend = Backend::new(terminal, self.render_config)?;
         self.prompt_with_backend(&mut backend)
     }
@@ -284,7 +337,7 @@ struct MultiSelectPrompt<'a, T> {
     help_message: Option<&'a str>,
     vim_mode: bool,
     cursor_index: usize,
-    checked: HashSet<usize>,
+    checked: BTreeSet<usize>,
     page_size: usize,
     keep_filter: bool,
     input: Input,
@@ -321,7 +374,7 @@ where
         let filtered_options = (0..mso.options.len()).collect();
         let checked_options = mso
             .default
-            .map_or_else(|| HashSet::new(), |d| d.iter().cloned().collect());
+            .map_or_else(|| BTreeSet::new(), |d| d.iter().cloned().collect());
 
         Ok(Self {
             message: mso.message,
@@ -370,7 +423,9 @@ where
         self.cursor_index = self.cursor_index.saturating_add(qty);
 
         if self.cursor_index >= self.filtered_options.len() {
-            self.cursor_index = if wrap {
+            self.cursor_index = if self.filtered_options.is_empty() {
+                0
+            } else if wrap {
                 self.cursor_index % self.filtered_options.len()
             } else {
                 self.filtered_options.len().saturating_sub(1)
@@ -457,14 +512,21 @@ where
         }
     }
 
-    fn retain_final_answer(&mut self) {
-        let mut index = 0;
-        let checked = self.checked.clone();
-        self.options.retain(|_| {
-            let keep = checked.contains(&index);
-            index += 1;
-            keep
-        });
+    fn get_final_answer(&mut self) -> Vec<ListOption<T>> {
+        let mut answer = vec![];
+
+        // by iterating in descending order, we can safely
+        // swap remove because the elements to the right
+        // that we did not remove will not matter anymore.
+        for index in self.checked.iter().rev() {
+            let index = *index;
+            let value = self.options.swap_remove(index);
+            let lo = ListOption::new(index, value);
+            answer.push(lo);
+        }
+        answer.reverse();
+
+        answer
     }
 
     fn render<B: MultiSelectBackend>(&mut self, backend: &mut B) -> InquireResult<()> {
@@ -508,7 +570,8 @@ where
             let key = backend.read_key()?;
 
             match key {
-                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Interrupt => interrupt_prompt!(),
+                Key::Cancel => cancel_prompt!(backend, &self.message),
                 Key::Submit => match self.validate_current_answer() {
                     Ok(()) => break,
                     Err(err) => self.error = Some(err),
@@ -517,20 +580,109 @@ where
             }
         }
 
-        self.retain_final_answer();
-
-        let final_answer: Vec<ListOption<T>> = self
-            .options
-            .into_iter()
-            .enumerate()
-            .map(|(idx, opt)| ListOption::new(idx, opt))
-            .collect();
-
+        let final_answer = self.get_final_answer();
         let refs: Vec<ListOption<&T>> = final_answer.iter().map(ListOption::as_ref).collect();
         let formatted = (self.formatter)(&refs);
 
-        backend.finish_prompt(&self.message, &formatted)?;
+        finish_prompt_with_answer!(backend, &self.message, &formatted, final_answer);
+    }
+}
 
-        Ok(final_answer)
+#[cfg(test)]
+#[cfg(feature = "crossterm")]
+mod test {
+    use crate::{
+        formatter::MultiOptionFormatter,
+        list_option::ListOption,
+        terminal::crossterm::CrosstermTerminal,
+        ui::{Backend, RenderConfig},
+        MultiSelect,
+    };
+    use crossterm::event::{KeyCode, KeyEvent};
+
+    #[test]
+    /// Tests that a closure that actually closes on a variable can be used
+    /// as a Select formatter.
+    fn closure_formatter() {
+        let read: Vec<KeyEvent> = vec![KeyCode::Char(' '), KeyCode::Enter]
+            .into_iter()
+            .map(KeyEvent::from)
+            .collect();
+        let mut read = read.iter();
+
+        let formatted = String::from("Thanks!");
+        let formatter: MultiOptionFormatter<i32> = &|_| formatted.clone();
+
+        let options = vec![1, 2, 3];
+
+        let mut write: Vec<u8> = Vec::new();
+        let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+        let mut backend = Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
+
+        let ans = MultiSelect::new("Question", options)
+            .with_formatter(formatter)
+            .prompt_with_backend(&mut backend)
+            .unwrap();
+
+        assert_eq!(vec![ListOption::new(0, 1)], ans);
+    }
+
+    #[test]
+    // Anti-regression test: https://github.com/mikaelmello/inquire/issues/30
+    fn down_arrow_on_empty_list_does_not_panic() {
+        let read: Vec<KeyEvent> = [
+            KeyCode::Char('9'),
+            KeyCode::Down,
+            KeyCode::Backspace,
+            KeyCode::Char('3'),
+            KeyCode::Down,
+            KeyCode::Backspace,
+            KeyCode::Enter,
+        ]
+        .iter()
+        .map(|c| KeyEvent::from(*c))
+        .collect();
+
+        let mut read = read.iter();
+
+        let options = vec![1, 2, 3];
+
+        let mut write: Vec<u8> = Vec::new();
+        let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+        let mut backend = Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
+
+        let ans = MultiSelect::new("Question", options)
+            .prompt_with_backend(&mut backend)
+            .unwrap();
+
+        assert_eq!(Vec::<ListOption<i32>>::new(), ans);
+    }
+
+    #[test]
+    // Anti-regression test: https://github.com/mikaelmello/inquire/issues/31
+    fn list_option_indexes_are_relative_to_input_vec() {
+        let read: Vec<KeyEvent> = vec![
+            KeyCode::Down,
+            KeyCode::Char(' '),
+            KeyCode::Down,
+            KeyCode::Char(' '),
+            KeyCode::Enter,
+        ]
+        .into_iter()
+        .map(KeyEvent::from)
+        .collect();
+        let mut read = read.iter();
+
+        let options = vec![1, 2, 3];
+
+        let mut write: Vec<u8> = Vec::new();
+        let terminal = CrosstermTerminal::new_with_io(&mut write, &mut read);
+        let mut backend = Backend::new(terminal, RenderConfig::default_static_ref()).unwrap();
+
+        let ans = MultiSelect::new("Question", options)
+            .prompt_with_backend(&mut backend)
+            .unwrap();
+
+        assert_eq!(vec![ListOption::new(1, 2), ListOption::new(2, 3)], ans);
     }
 }
