@@ -2,7 +2,8 @@ use crate::{
     error::{InquireError, InquireResult},
     formatter::StringFormatter,
     input::Input,
-    ui::{crossterm::CrosstermTerminal, Backend, Key, KeyModifiers, PasswordBackend, RenderConfig},
+    terminal::get_default_terminal,
+    ui::{Backend, Key, KeyModifiers, PasswordBackend, RenderConfig},
     validator::StringValidator,
 };
 
@@ -45,12 +46,18 @@ pub enum PasswordDisplayMode {
 /// # Example
 ///
 /// ```no_run
-///  use inquire::{min_length, validator::InquireLength, Password, PasswordDisplayMode};
+///  use inquire::{validator::StringValidator, Password, PasswordDisplayMode};
 ///
-///  let name = Password::new("RSA Encryption Key:")
+///  let validator: StringValidator = &|input| if input.chars().count() < 10 {
+///      Err(String::from("Keys must have at least 10 characters."))
+///  } else {
+///      Ok(())
+///  };
+///
+///  let name = Password::new("Encryption Key:")
 ///      .with_display_toggle_enabled()
 ///      .with_display_mode(PasswordDisplayMode::Hidden)
-///      .with_validator(min_length!(10))
+///      .with_validator(validator)
 ///      .with_formatter(&|_| String::from("Input received"))
 ///      .with_help_message("It is recommended to generate a new one only for this purpose")
 ///      .prompt();
@@ -86,6 +93,13 @@ pub struct Password<'a> {
     pub validators: Vec<StringValidator<'a>>,
 
     /// RenderConfig to apply to the rendered interface.
+    ///
+    /// Note: The default render config considers if the NO_COLOR environment variable
+    /// is set to decide whether to render the colored config or the empty one.
+    ///
+    /// When overriding the config in a prompt, NO_COLOR is no longer considered and your
+    /// config is treated as the only source of truth. If you want to customize colors
+    /// and still suport NO_COLOR, you will have to do this on your end.
     pub render_config: &'a RenderConfig,
 }
 
@@ -169,15 +183,39 @@ impl<'a> Password<'a> {
     }
 
     /// Sets the provided color theme to this prompt.
+    ///
+    /// Note: The default render config considers if the NO_COLOR environment variable
+    /// is set to decide whether to render the colored config or the empty one.
+    ///
+    /// When overriding the config in a prompt, NO_COLOR is no longer considered and your
+    /// config is treated as the only source of truth. If you want to customize colors
+    /// and still suport NO_COLOR, you will have to do this on your end.
     pub fn with_render_config(mut self, render_config: &'a RenderConfig) -> Self {
         self.render_config = render_config;
         self
     }
 
+    /// Parses the provided behavioral and rendering options and prompts
+    /// the CLI user for input according to the defined rules.
+    ///
+    /// This method is intended for flows where the user skipping/cancelling
+    /// the prompt - by pressing ESC - is considered normal behavior. In this case,
+    /// it does not return `Err(InquireError::OperationCanceled)`, but `Ok(None)`.
+    ///
+    /// Meanwhile, if the user does submit an answer, the method wraps the return
+    /// type with `Some`.
+    pub fn prompt_skippable(self) -> InquireResult<Option<String>> {
+        match self.prompt() {
+            Ok(answer) => Ok(Some(answer)),
+            Err(InquireError::OperationCanceled) => Ok(None),
+            Err(err) => Err(err),
+        }
+    }
+
     /// Parses the provided behavioral and rendering options and prompts
     /// the CLI user for input according to the defined rules.
     pub fn prompt(self) -> InquireResult<String> {
-        let terminal = CrosstermTerminal::new()?;
+        let terminal = get_default_terminal()?;
         let mut backend = Backend::new(terminal, self.render_config)?;
         self.prompt_with_backend(&mut backend)
     }
@@ -296,7 +334,8 @@ impl<'a> PasswordPrompt<'a> {
             let key = backend.read_key()?;
 
             match key {
-                Key::Cancel => return Err(InquireError::OperationCanceled),
+                Key::Interrupt => interrupt_prompt!(),
+                Key::Cancel => cancel_prompt!(backend, &self.message),
                 Key::Submit => match self.get_final_answer() {
                     Ok(answer) => {
                         final_answer = answer;
@@ -308,18 +347,21 @@ impl<'a> PasswordPrompt<'a> {
             }
         }
 
-        backend.finish_prompt(&self.message, &(self.formatter)(&final_answer))?;
+        let formatted = (self.formatter)(&final_answer);
 
-        Ok(final_answer)
+        finish_prompt_with_answer!(backend, &self.message, &formatted, final_answer);
     }
 }
 
 #[cfg(test)]
+#[cfg(feature = "crossterm")]
 mod test {
     use super::Password;
-    use crate::ui::{crossterm::CrosstermTerminal, Backend, RenderConfig};
+    use crate::{
+        terminal::crossterm::CrosstermTerminal,
+        ui::{Backend, RenderConfig},
+    };
     use crossterm::event::{KeyCode, KeyEvent};
-    use ntest::timeout;
 
     fn default<'a>() -> Password<'a> {
         Password::new("Question?")
@@ -338,7 +380,6 @@ mod test {
 
         ($name:ident,$input:expr,$output:expr,$prompt:expr) => {
             #[test]
-            #[timeout(100)]
             fn $name() {
                 let read: Vec<KeyEvent> = $input.into_iter().map(KeyEvent::from).collect();
                 let mut read = read.iter();
